﻿//*************************************************************************************************
//【サンプルプログラム】『Kinectのランダムドットパターンで赤外線脈波を検出』
//【名称】Kinect_HeartRateMonitor　　【Version】SDK Ver.1.6対応版
//【作成者】私設研究所Neo-Tech-Lab.com  上田智章  Copyright by ☆Tomoaki Ueda☆
//【作成日】2013年01月14日
//*************************************************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Media;                   //【SoundPlayer】
using System.Diagnostics;             //【Debug】
using System.Drawing;                 //【Bitmap】【Color】
using System.Drawing.Imaging;         //【BitmapData】
using System.Runtime.InteropServices; //【Marshal.Copy】
using Microsoft.Kinect;               //【参照設定】プロジェクト(P)⇒参照の追加(F)⇒.NETタブ⇒Microsoft.Kinectを選択

namespace Kinect_VitalMonitor3
{
    public partial class Form1 : Form
    {
        //【KINECT関連】
        //　関数内でダイナミックに変数を確保すると動作速度が遅くなるので、ここにまとめて変数を確保しています。
        private KinectSensor KS_KinectSensor;     //【KINECTセンサ】           ★利用可能なKinectセンサ

        //【赤外線カメラ関連】
        private ColorImageFrame KS_ColorFrame;    //【KINECTのColorImageFrame】★赤外線カメラ画像取得に使う
        private byte[] KS_InfraredImageData;      //【KINECTのColorImageData】16bit/画素,GrayScale表示形式
        private byte[] KS_ColorImageDataXRGB;     //【KINECTのColorImageData】32bit/画素,X8R8G8B8表示形式
        private int KS_ColorFrameWidth;           //【KINECTのColorFrameの横方向画素数】
        private int KS_ColorFrameHeight;          //【KINECTのColorFrameの縦方向画素数】
        private Bitmap KS_bmp1;                   //【ColorImage表示用Bitmapオブジェクト】
        private BitmapData KS_data1;              //【ColorImage用BitmapData】
        private Rectangle KS_rect1;               //【ColorImage表示枠】

        //【デプスカメラ関連】
        private DepthImageFrame KS_DepthFrame;    //【KINECTのDepthImageFrame】
        private short[] KS_DepthValue;            //
        private int KS_DepthFrameWidth;           //【KINECTのDepthFrameの横方向画素数】
        private int KS_DepthFrameHeight;          //【KINECTのDepthFrameの縦方向画素数】

        //【NTL拡張現実センサ関連】---------------------------------------------------------
        private NTL_ColorBar InfraredColorBar = new NTL_ColorBar(); //【赤外線強度をカラーに変換する部品】
        private NTL_ColorBar DepthColorBar = new NTL_ColorBar();    //【デプス値をカラーに変換する部品】

        //【NTL拡張現実センサ関連】---------------------------------------------------------
        private int KS_HeadAngle = 15;          //【Kinectヘッドの仰角を設定】設定範囲は±27°

        //【NTL拡張現実センサ関連】---------------------------------------------------------
        private Rectangle NTL_RectHeartRate = new Rectangle(290, 210, 60, 60); //【脈拍観測枠】
        private Rectangle NTL_RectBreathing = new Rectangle(220, 410, 50, 50); //【呼吸観測枠】
        private int NTL_Threshold = 10000; //【赤外線しきい値】
        private int NTL_Count = 0;   //【計測回数】
        private int NTL_ACount = 0;  //【加速度センサ用計測回数】
        private int NTL_DCount = 0;  //【デプス計測回数】

        private const int NTL_nFIFO = 601;               //【波形表示の為のデータ格納配列サイズ】
        private float[] NTL_FIFO = new float[NTL_nFIFO]; //【波形表示の為のデータ格納用FIFO】
        private int NTL_ptr = 0;                         //【FIFOのポインタ】
        private System.Drawing.Point[] NTL_Points = new System.Drawing.Point[NTL_nFIFO];
        private float NTL_maxWave, NTL_minWave;

        private float[] NTL_Depth = new float[NTL_nFIFO]; //【波形表示の為のデータ格納用FIFO】
        private int NTL_pDepth = 0;                         //【FIFOのポインタ】
        private System.Drawing.Point[] NTL_DepthPoints = new System.Drawing.Point[NTL_nFIFO];
        private float NTL_maxDepth, NTL_minDepth;

        private const int NTL_nWave = 24;                //
        private float NTL_n1Wave;                        //
        private int NTL_pWave0, NTL_pWave1, NTL_pWave2;  //【ポインタ】
        private float[] NTL_Wave = new float[NTL_nWave]; //
        private float NTL_sWave;                         //

        private const int NTL_nLPF = 12;                 //
        private float NTL_n1LPF;                         //
        private int NTL_pLPF;                            //
        private float[] NTL_LPF = new float[NTL_nLPF];   //
        private float NTL_sLPF;                          //

        private const int NTL_nBrLPF = 30;                 //
        private float NTL_n1BrLPF;                         //
        private int NTL_pBrLPF;                            //
        private float[] NTL_BrLPF = new float[NTL_nBrLPF]; //
        private float NTL_sBrLPF;                          //


        public Form1()
        {
            InitializeComponent();

            //***********************************************************
            //【Kinect検出】【共通】
            //***********************************************************
            //【利用可能なKINECTセンサを取得する】
            KS_KinectSensor = (from sensorToCheck
                               in KinectSensor.KinectSensors 
                               where sensorToCheck.Status == KinectStatus.Connected
                               select sensorToCheck).FirstOrDefault();
            //【Kinect検出結果の表示】
            if (KS_KinectSensor == null) //【Kinect接続が確認できなかった場合】
            {
                pictureBox2.Image = imageList1.Images[1];
                MessageBox.Show(
                   "接続可能なKINECTセンサが見当たりません。",
                   "KINECT for Windows Error Message",
                   MessageBoxButtons.OK
                );
                return;
            }
            else //【Kinect検出に成功した場合】
            {
                pictureBox2.Image = imageList1.Images[0];
            }

            //***********************************************************
            //【Kinectの初期化処理】
            //***********************************************************
            KS_KinectSensor.ColorStream.Enable(ColorImageFormat.InfraredResolution640x480Fps30);  //【InfraredStream許可】
            KS_KinectSensor.DepthStream.Enable(DepthImageFormat.Resolution640x480Fps30);          //【DepthStream許可】
            KS_KinectSensor.Start();                                                              //【KINECT動作開始】

            //【赤外線カメラ関連】
            KS_ColorFrameWidth = KS_KinectSensor.ColorStream.FrameWidth;                          //【ColorImageの横方向画素数】
            KS_ColorFrameHeight = KS_KinectSensor.ColorStream.FrameHeight;                        //【ColorImageの縦方向画素数】
            KS_ColorImageDataXRGB = new byte[2*KS_KinectSensor.ColorStream.FramePixelDataLength]; //【カラー表示用フレームメモリを確保】
            KS_InfraredImageData = new byte[KS_KinectSensor.ColorStream.FramePixelDataLength];    //
            KS_bmp1 = new Bitmap(KS_ColorFrameWidth, KS_ColorFrameHeight, System.Drawing.Imaging.PixelFormat.Format32bppRgb); //【ColorImage表示用】
            KS_rect1 = new Rectangle(0, 0, KS_ColorFrameWidth, KS_ColorFrameHeight);              //【ColorImage用枠】
            pictureBox1.SizeMode = PictureBoxSizeMode.StretchImage;                               //【キャプチャー画像をpictureBox1枠内に表示】

            //【デプス関連】
            KS_DepthFrameWidth = KS_KinectSensor.DepthStream.FrameWidth;   //【DepthImageの横方向画素数】
            KS_DepthFrameHeight = KS_KinectSensor.DepthStream.FrameHeight; //【DepthImageの縦方向画素数】
            KS_DepthValue = new short[KS_DepthFrameWidth * KS_DepthFrameHeight];
            KS_KinectSensor.DepthStream.Range = DepthRange.Near;    //【Depth測定モードをNearに設定】

            KS_KinectSensor.ElevationAngle = KS_HeadAngle; //【Kinectの仰角を変更する】
            textBox1.Text = KS_HeadAngle.ToString();       //

            InfraredColorBar.CreateColorPalletForInfrared(65536);           //【赤外線強度16bitをカラー変換】
            pictureBox3.Image = InfraredColorBar.CreateColorScale(400, 10); //【カラーバーを400×10画素で作成して表示する】
            pictureBox3.SizeMode = PictureBoxSizeMode.StretchImage;         //【カラーバーをpictureBox3枠内に表示】

            KS_KinectSensor.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(ColorImageReady); //【KINECTデータ取得イベントとしてColorImageReadyを設定】
            KS_KinectSensor.DepthFrameReady += new EventHandler<DepthImageFrameReadyEventArgs>(DepthImageReady); //【DepthImageReadyをcallするよう設定】
            this.FormClosing += new FormClosingEventHandler(Form1_FormClosing);     //【Kinectの終了処理】
            checkBox2.CheckedChanged += new EventHandler(checkBox2_CheckedChanged); //【赤外線プロジェクタの照射のOn/Offを切り替える】
        }

        //***************************************************************
        //【Kinectの終了処理】
        //***************************************************************
        void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (KS_KinectSensor != null)
            {
                KS_KinectSensor.Stop();    //【KINECTセンサを終了】★これを実行しないとレーザーが消えない
                KS_KinectSensor.Dispose(); //【KINECTセンサのオブジェクトを解放する】
            }
        }

        //***************************************************************
        //【ColorImageの取得イベント】
        //***************************************************************
        // Kinectからカラー画像１フレーム分が到着する度に、この関数が呼ばれます。
        // Windowsパソコンが最も高速にBitmapデータを表示できるデータ形式は1画素４バイトの形式なのでXRGB形式を使っています。
        // 但し、追加のCG描画で半透明表現を使うときはARGB形式を使います。
        unsafe private void ColorImageReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            int i,j;
            float Data, Dx, Dy;

            using (KS_ColorFrame = e.OpenColorImageFrame())
            {
                if (KS_ColorFrame != null)
                {
                    //【赤外線カメラ画像の取得と表示】
                    KS_ColorFrame.CopyPixelDataTo(KS_InfraredImageData); //【KINECTのColor画像を配列KS_InfraredImageDataにコピー】
                    KS_ConvertInfraredToColor();
                    KS_data1 = KS_bmp1.LockBits(KS_rect1, ImageLockMode.WriteOnly, System.Drawing.Imaging.PixelFormat.Format32bppRgb); //【Lock】
                    Marshal.Copy(KS_ColorImageDataXRGB, 0, KS_data1.Scan0, KS_ColorImageDataXRGB.Length); //【UnmanagedMemoryのBitmapDataに配列PixelDataをコピー】
                    KS_bmp1.UnlockBits(KS_data1);                                           //【Lock解除】
                    pictureBox1.Image = KS_bmp1;                                            //【pictureBox1に画像表示】

                    Data = GetRectangleInfraredIntensity(NTL_RectHeartRate); //【距離測定】指定領域内の加算平均デプス値を求める

                    if (NTL_Count == 0) //【初回はFIFOを初期化する】
                    {
                        InitializeRectangleWaveFilter(); //【矩形波相関フィルタの初期化を行う】
                        InitializeLowPassFilter();       //【LPFの初期化】
                        InitializeFIFO();                //【FIFOの初期化】
                    }
                    Data = GetRectangleWaveFilter(Data); //【矩形波相関フィルタ処理】
                    Data = GetLowPassFilter(Data);       //【LPF】
                    SetFIFO(Data);                       //【FIFO】

                    //【距離測定している矩形領域をデプス画像上に表示する】
                    Graphics g = Graphics.FromImage(pictureBox1.Image);    //【pixtureBox1内に測定値を重畳表示】
                    Color lc = Color.Azure;                                //【領域位置表示色の定義】
                    g.DrawRectangle(new Pen(lc, 3.0f), NTL_RectHeartRate); //【観測領域】
                    lc = Color.Yellow;                                     //【領域位置表示色の定義】
                    g.DrawRectangle(new Pen(lc, 3.0f), NTL_RectBreathing); //【観測領域】
                    g.DrawLine(new Pen(lc, 3.0f), 320, 0, 320, 480);       //【画面中央縦線】
                    g.DrawLine(new Pen(lc, 3.0f), 0, 240, 640, 240);       //【画面中央横線】

                    lc = Color.Pink;
                    Dx = pictureBox1.Width / NTL_nFIFO;
                    Dy = pictureBox1.Height / (NTL_maxWave - NTL_minWave);
                    for (i = 0, j = NTL_ptr; i < NTL_nFIFO; i++)
                    {
                        NTL_Points[i].X = (int)(i * Dx);
                        NTL_Points[i].Y = (int)((NTL_FIFO[j] - NTL_minWave) * Dy);
                        j++; if (j == NTL_nFIFO) { j = 0; } //【FIFO管理ポインタを更新する】
                    }
                    g.DrawLines(new Pen(lc, 5.0f), NTL_Points);

                    if (NTL_DCount > 0)
                    {
                        lc = Color.LightGreen;
                        Dx = pictureBox1.Width / NTL_nFIFO;
                        Dy = pictureBox1.Height / (NTL_maxDepth - NTL_minDepth);
                        for (i = 0, j = NTL_pDepth; i < NTL_nFIFO; i++)
                        {
                            NTL_DepthPoints[i].X = (int)(i * Dx);
                            NTL_DepthPoints[i].Y = (int)((NTL_Depth[j] - NTL_minDepth) * Dy);
                            j++; if (j == NTL_nFIFO) { j = 0; } //【FIFO管理ポインタを更新する】
                        }
                        g.DrawLines(new Pen(lc, 5.0f), NTL_DepthPoints);
                    }

                    NTL_Count++; //【測定番号を+1する】
 
                    DisplayLabels();       //【数値表示】6フレームに1回表示
                }
                else
                {
                    //【ImageFrameがnullだったのは要求が間に合わなかったと考えられます】
                }
            }
        }

        //***************************************************************
        //【ConvertInfraredToColor】16ビットのInfrared値をカラーバーに対応する色に変更
        //***************************************************************
        private void KS_ConvertInfraredToColor()
        {
            int i, j, L = 0, data;
            for (i = 0; i < KS_InfraredImageData.Length; i += 2)
            {
                data = KS_InfraredImageData[i + 1] * 256 + KS_InfraredImageData[i]; //【IR値16bitからパレット番号取得】
                if (data > 65535) { data = 65535; }
                j = data * 4;
                KS_ColorImageDataXRGB[L] = InfraredColorBar.NTL_ColorElement[j];
                KS_ColorImageDataXRGB[L + 1] = InfraredColorBar.NTL_ColorElement[j + 1];
                KS_ColorImageDataXRGB[L + 2] = InfraredColorBar.NTL_ColorElement[j + 2];
                KS_ColorImageDataXRGB[L + 3] = InfraredColorBar.NTL_ColorElement[j + 3];
                L += 4;
            }
        }

        //***************************************************************
        //【GetRectangleInfraredIntensity】
        //　閾値NTL_Thresholdを超えた赤外線強度サンプルの領域内平均値を得る
        //***************************************************************
        private float GetRectangleInfraredIntensity(Rectangle rect)
        {
            float sum = 0.0f;
            int i, j, k, L, n=0, v;
            int x0 = rect.Location.X; 
            int y0 = rect.Location.Y;
            int w = rect.Size.Width;
            int h = rect.Size.Height;
            for (j = y0; j < y0 + h; j++)
            {
                k = KS_ColorFrameWidth * j;
                for (i = x0; i < x0 + w; i++)
                {
                    L = 2 * (i + k);
                    v = (int)KS_InfraredImageData[L + 1] * 256 + (int)KS_InfraredImageData[L];
                    if (v > NTL_Threshold) { sum += v; n++; }
                }
            }
            if (n == 0) { n = 1; }
            sum /= (float)(n);
            return sum;
        }

        //***************************************************************
        //【矩形波相関フィルタ】★初期化
        //***************************************************************
        private void InitializeRectangleWaveFilter()
        {
            NTL_pWave0 = 0; NTL_pWave1 = 6; NTL_pWave2 = 18;
            NTL_n1Wave = 1.0f / (float)NTL_nWave;
            NTL_sWave = 0.0f;
            for (int i = 0; i < NTL_nWave; i++) { NTL_Wave[i] = 0.0f; }
        }

        //***************************************************************
        //【矩形波相関フィルタ】
        //***************************************************************
        private float GetRectangleWaveFilter(float d)
        {
            NTL_sWave += -d + 2 * (NTL_Wave[NTL_pWave2] - NTL_Wave[NTL_pWave1]) + NTL_Wave[NTL_pWave0];
            NTL_Wave[NTL_pWave0] = d;
            NTL_pWave0++; if (NTL_pWave0 == NTL_nWave) { NTL_pWave0 = 0; }
            NTL_pWave1++; if (NTL_pWave1 == NTL_nWave) { NTL_pWave1 = 0; }
            NTL_pWave2++; if (NTL_pWave2 == NTL_nWave) { NTL_pWave2 = 0; }
            return NTL_sWave * NTL_n1Wave;
        }

        //***************************************************************
        //【LPFフィルタ】★初期化
        //***************************************************************
        private void InitializeLowPassFilter()
        {
            NTL_pLPF = 0;
            NTL_n1LPF = 1.0f / NTL_nLPF;
            NTL_sLPF = 0.0f;
            for (int i = 0; i < NTL_nLPF; i++) { NTL_LPF[i] = 0.0f; }
        }

        //***************************************************************
        //【LPFフィルタ】
        //***************************************************************
        private float GetLowPassFilter(float d)
        {
            NTL_sLPF += d - NTL_LPF[NTL_pLPF];
            NTL_LPF[NTL_pLPF] = d;
            NTL_pLPF++; if (NTL_pLPF == NTL_nLPF) { NTL_pLPF = 0; }
            return NTL_sLPF * NTL_n1LPF;
        }

        //***************************************************************
        //【FIFO】★初期化
        //***************************************************************
        private void InitializeFIFO()
        {
            NTL_ptr = 0; //【ポインタ初期化】
            for (int i = 0; i < NTL_nFIFO; i++) { NTL_FIFO[i] = 0.0f; }
        }

        //***************************************************************
        //【FIFO】
        //***************************************************************
        private void SetFIFO(float d)
        {
            //【FIFO】
            NTL_FIFO[NTL_ptr] = d;                         //【波形表示用の測定値を保存する】
            NTL_ptr++; if (NTL_ptr == NTL_nFIFO) { NTL_ptr = 0; } //【FIFO管理ポインタを更新する】

            //【FIFO内の最大値と最小値を求める】
            NTL_minWave = 800000.0f;
            NTL_maxWave = 0.0f;
            for (int i = 0; i < NTL_nFIFO; i++)
            {
                if (NTL_FIFO[i] > NTL_maxWave) NTL_maxWave = NTL_FIFO[i];
                if (NTL_FIFO[i] < NTL_minWave) NTL_minWave = NTL_FIFO[i];
            }
            if (NTL_maxWave == NTL_minWave) { NTL_maxWave = NTL_minWave + 2.0f; }
        }

        //***************************************************************
        //【指定フレーム数毎に最近点座標とKionix３軸直交加速度センサの測定値を表示】
        //***************************************************************
        private void DisplayLabels()
        {
            NTL_ACount++;        //【表示用カウンタを+1】
            if (NTL_ACount == 6) //【30FPSなら0.2秒に1回表示】
            {
                NTL_ACount = 0;  //【表示用カウンタをクリア】
                Vector4 Acc = KS_KinectSensor.AccelerometerGetCurrentReading();
                label9.Text  = "x=" + Acc.X.ToString() + "[g]";
                label10.Text = "y=" + Acc.Y.ToString() + "[g]";
                label11.Text = "z=" + Acc.Z.ToString() + "[g]";
                label12.Text = "|Acc|=" + Math.Sqrt(Acc.X * Acc.X + Acc.Y * Acc.Y + Acc.Z * Acc.Z).ToString() + "[g]";
            }
        }

        //***************************************************************
        //【上下角変更ボタン】KINECTの上下角を変更。±27°の範囲内。時間を置かないとエラーが発生する
        //***************************************************************
        private void button2_Click(object sender, EventArgs e)
        {
            Int32 angle = int.Parse(textBox1.Text); //【設定角度を読み込む】±27°
            if (angle <= KS_KinectSensor.MaxElevationAngle && angle >= KS_KinectSensor.MinElevationAngle)
            {
                KS_HeadAngle = angle;
                KS_KinectSensor.ElevationAngle = KS_HeadAngle; //【上下角の変更】
            }
        }

        //***************************************************************
        //【赤外線プロジェクタの照射のOn/Offを切り替える】
        //***************************************************************
        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox2.Checked) { KS_KinectSensor.ForceInfraredEmitterOff = true; }
            else { KS_KinectSensor.ForceInfraredEmitterOff = false; }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            //【再計測の処理はNTL_Countを0にするだけで開始する】
            NTL_Count = 0;
        }


        //***************************************************************
        //【DepthImageの取得イベント】
        //***************************************************************
        private void DepthImageReady(object sender, DepthImageFrameReadyEventArgs e)
        {
            using (KS_DepthFrame = e.OpenDepthImageFrame())
            {
                if (KS_DepthFrame != null)
                {
                    float d;
                    KS_DepthFrame.CopyPixelDataTo(KS_DepthValue);     //【KINECTのDepth情報を配列DepthDataにコピー】
                    if (NTL_DCount == 0) {
                        InitializeDepth();
                        InitializeBreathingLowPassFilter();
                    }
                    d = GetRectangleDepthAverage(NTL_RectBreathing);  //【指定矩形枠内の奥行きデータ平均値】
                    d = GetBreathingLowPassFilter(d);
                    SetDepth(d);
                    NTL_DCount++;
                }
                else
                {
                    //【DepthFrameがnullだったのは要求が間に合わなかったと考えられます】
                }
            }
        }

        //***************************************************************
        //【FIFO】★初期化
        //***************************************************************
        private void InitializeDepth()
        {
            NTL_pDepth = 0; //【ポインタ初期化】
            for (int i = 0; i < NTL_nFIFO; i++) { NTL_Depth[i] = 0.0f; }
        }

        //***************************************************************
        //【FIFO】
        //***************************************************************
        private void SetDepth(float d)
        {
            //【FIFO】
            NTL_Depth[NTL_pDepth] = d;                         //【波形表示用の測定値を保存する】
            NTL_pDepth++; if (NTL_pDepth == NTL_nFIFO) { NTL_pDepth = 0; } //【FIFO管理ポインタを更新する】

            //【FIFO内の最大値と最小値を求める】
            NTL_minDepth = 800000.0f;
            NTL_maxDepth = 0.0f;
            for (int i = 0; i < NTL_nFIFO; i++)
            {
                if (NTL_Depth[i] > NTL_maxDepth) NTL_maxDepth = NTL_Depth[i];
                if (NTL_Depth[i] < NTL_minDepth) NTL_minDepth = NTL_Depth[i];
            }
            if (NTL_maxDepth == NTL_minDepth) { NTL_maxDepth = NTL_minDepth + 2.0f; }
        }

        //***************************************************************
        //【GetRectangleInfraredIntensity】
        //　閾値NTL_Thresholdを超えた赤外線強度サンプルの領域内平均値を得る
        //***************************************************************
        private float GetRectangleDepthAverage(Rectangle rect)
        {
            float sum = 0.0f;
            int i, j, k, n = 0, v;
            int x0 = rect.Location.X;
            int y0 = rect.Location.Y;
            int w = rect.Size.Width;
            int h = rect.Size.Height;
            for (j = y0; j < y0 + h; j++)
            {
                k = KS_DepthFrameWidth * j;
                for (i = x0; i < x0 + w; i++)
                {
                    v = (int)(KS_DepthValue[i+k] >> DepthImageFrame.PlayerIndexBitmaskWidth);
                    if (v > KS_DepthFrame.MinDepth) { sum += v; n++; }
                }
            }
            if (n == 0) { n = 1; }
            sum /= (float)(n);
            return sum;
        }

        //***************************************************************
        //【LPFフィルタ】★初期化
        //***************************************************************
        private void InitializeBreathingLowPassFilter()
        {
            NTL_pBrLPF = 0;
            NTL_n1BrLPF = 1.0f / NTL_nBrLPF;
            NTL_sBrLPF = 0.0f;
            for (int i = 0; i < NTL_nBrLPF; i++) { NTL_BrLPF[i] = 0.0f; }
        }

        //***************************************************************
        //【LPFフィルタ】
        //***************************************************************
        private float GetBreathingLowPassFilter(float d)
        {
            NTL_sBrLPF += d - NTL_BrLPF[NTL_pBrLPF];
            NTL_BrLPF[NTL_pBrLPF] = d;
            NTL_pBrLPF++; if (NTL_pBrLPF == NTL_nBrLPF) { NTL_pBrLPF = 0; }
            return NTL_sBrLPF * NTL_n1BrLPF;
        }

    }
}
